@@ -6,14 +6,15 @@
 #define EVDI_LIB_H
 
 #include <stdbool.h>
+#include <stdint.h>
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
 #define LIBEVDI_VERSION_MAJOR 1
-#define LIBEVDI_VERSION_MINOR 12
-#define LIBEVDI_VERSION_PATCH 0
+#define LIBEVDI_VERSION_MINOR 14
+#define LIBEVDI_VERSION_PATCH 2
 
 struct evdi_lib_version {
 	int version_major;
@@ -106,6 +107,9 @@ evdi_handle evdi_open_attached_to(const char *sysfs_parent_device);
 
 void evdi_close(evdi_handle handle);
 void evdi_connect(evdi_handle handle, const unsigned char *edid,
+		  const unsigned int edid_length,
+		  const uint32_t sku_area_limit);
+void evdi_connect2(evdi_handle handle, const unsigned char *edid,
 		  const unsigned int edid_length,
 		  const uint32_t pixel_area_limit,
 		  const uint32_t pixel_per_second_limit);
