@@ -10,11 +10,11 @@
 #include "linux/thread_info.h"
 #include "linux/mm.h"
 #include <linux/version.h>
-#if KERNEL_VERSION(5, 16, 0) <= LINUX_VERSION_CODE
+#if KERNEL_VERSION(5, 16, 0) <= LINUX_VERSION_CODE || defined(EL8) || defined(EL9)
 #include <drm/drm_file.h>
 #include <drm/drm_vblank.h>
 #include <drm/drm_ioctl.h>
-#elif KERNEL_VERSION(5, 5, 0) <= LINUX_VERSION_CODE || defined(EL8)
+#elif KERNEL_VERSION(5, 5, 0) <= LINUX_VERSION_CODE
 #else
 #include <drm/drmP.h>
 #endif
@@ -31,7 +31,7 @@
 
 #include <linux/dma-buf.h>
 
-#if KERNEL_VERSION(5, 16, 0) <= LINUX_VERSION_CODE
+#if KERNEL_VERSION(5, 16, 0) <= LINUX_VERSION_CODE || defined(EL9)
 MODULE_IMPORT_NS(DMA_BUF);
 #endif
 
@@ -727,7 +727,7 @@ void evdi_painter_dpms_notify(struct evdi_device *evdi, int mode)
 static void evdi_log_pixel_format(uint32_t pixel_format,
 		char *buf, size_t size)
 {
-#if KERNEL_VERSION(5, 14, 0) <= LINUX_VERSION_CODE
+#if KERNEL_VERSION(5, 14, 0) <= LINUX_VERSION_CODE || defined(EL8)
 	snprintf(buf, size, "pixel format %p4cc", &pixel_format);
 #else
 	struct drm_format_name_buf format_name;
@@ -749,13 +749,16 @@ void evdi_painter_mode_changed_notify(struct evdi_device *evdi,
 	if (painter == NULL)
 		return;
 
+	painter_lock(painter);
 	fb = &painter->scanout_fb->base;
-	if (fb == NULL)
+	if (fb == NULL) {
+		painter_unlock(painter);
 		return;
+	}
 
 	bits_per_pixel = fb->format->cpp[0] * 8;
 	pixel_format = fb->format->format;
-
+	painter_unlock(painter);
 
 	evdi_log_pixel_format(pixel_format, buf, sizeof(buf));
 	EVDI_INFO("(card%d) Notifying mode changed: %dx%d@%d; bpp %d; %s",
@@ -885,7 +888,7 @@ evdi_painter_connect(struct evdi_device *evdi,
 
 	painter_lock(painter);
 
-        evdi->pixel_area_limit = pixel_area_limit;
+	evdi->pixel_area_limit = pixel_area_limit;
 	evdi->pixel_per_second_limit = pixel_per_second_limit;
 	painter->drm_filp = file;
 	kfree(painter->edid);
@@ -1217,6 +1220,7 @@ void evdi_painter_cleanup(struct evdi_painter *painter)
 
 	painter->drm_device = NULL;
 	painter_unlock(painter);
+	kfree(painter);
 }
 
 void evdi_painter_set_scanout_buffer(struct evdi_painter *painter,
